@@ -11,5 +11,5 @@ soundgen_app = function() {
     stop("Could not find example directory. Try re-installing `soundgen`.",
          call. = FALSE)
   }
-  shiny::runApp(appDir, display.mode = "normal")
+  shiny::runApp(appDir, display.mode = "normal", launch.browser = TRUE)
 }
